/**********************************************************************
  Copyright (c) 2007-2008, Space Machines Laboratory, Tohoku University
  Copyright (c) 2007-2008, PieCake,Inc.
**********************************************************************/

#ifndef __ENGINE_REDUNDANCY_H__
#define __ENGINE_REDUNDANCY_H__


// C++ & C
#include <string>
#include <vector>


// OpenRTM
#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>


// OpenHRP
#include "hrpModelHeaders.h"


// USER
#include "ConfigSkel.h"
#include "standardModelHeaders.h"


// namespace
using namespace std;
using namespace OpenHRP;
using namespace RTC;
using namespace tohoku;


// MACRO
#define DEG2RAD(x)  ((x)/180.0*M_PI)
#define RAD2DEG(x)  ((x)/M_PI*180.0)



class EngineRedundancy
{
public:
  EngineRedundancy();
  virtual ~EngineRedundancy();


  //-----------------------------------------------
  //  function
  //-----------------------------------------------

  // for initialize
  bool init(JointName* in_jointname);
  void finish();
  bool isInit() { return is_init; }


  // main calc function
  bool update(short mode=0, RightLeft type=RIGHT, CORBA::Boolean isHoldOtherArm=0);


  // for limitter
  void setProperty(const double dt, const double limitPerSec);




  ////////  for current data  ////////
  // set current data
  bool setCurAngle(const TimedDoubleSeq q);

  // get current data
  TimedDoubleSeq getCurPosition();
  TimedDoubleSeq getCurRedundant();
  double getCurElbowAngle(RightLeft type=RIGHT);




  ////////  for reference data  ////////
  // set reference data
  bool setRefPosition(const TimedDoubleSeq p);
  bool setRefRedundant(const TimedDoubleSeq phi);
  
  // get reference data
  TimedDoubleSeq getResultAngle();
  double getResultElbowAngle(RightLeft type=RIGHT);




  ////////  for service port  ////////
  bool resetOriginalFrame();
  bool selectRedundancy(const RedundantAxesMask& redundancy);
  bool setControlPointOffset(RightLeft type, const HgMatrix offset);


  //-----------------------------------------------
  //  variable
  //-----------------------------------------------

  enum PATHTYPE {ALL, ARM};
  enum Direction {Right, Left, Upper, Lower};


  // for status port
  double condition_number;


private:
  //-----------------------------------------------
  //  function
  //-----------------------------------------------
  // for status port
  double SVD(const dmatrix &J);


  // for inverse kinematics
  dvector calcDeltaP(const vector3 &p_ref_in, const matrix33 &R_ref_in, RightLeft type=RIGHT);
  bool SCmethod(dvector &d_theta, const dmatrix &Jac, const dvector6 &d_x_ref) const;

  bool calcMinimizeNorm(const vector3 p_ref, const matrix33 R_ref, RightLeft type=RIGHT, bool withWaist=false, bool withMove=false);
  bool calcElbowAngle(const vector3 p_ref, const matrix33 R_ref, const double elbow_ref, RightLeft type);
  bool calcMinimizeNormDual(const vector3 p_ref_r, const matrix33 R_ref_r, const vector3 p_ref_l, const matrix33 R_ref_l, bool withWaist=false, bool withMove=false);


  bool checkResults(RightLeft type, bool withElbow, bool withComment=false);


  // matrix tools
  dvector3 cross_prod(dvector3 &a, dvector3 &b);
  dmatrix  diag(const dmatrix &in1, const dmatrix &in2);
  dmatrix  addToMat(const dmatrix &in1, const dmatrix &in2, const Direction dir=Lower);



  //-----------------------------------------------
  //  variable
  //-----------------------------------------------

  // joint name
  JointName *jointname;


  // 一秒あたりの最大変化量と周期
  // ユーザーがプロパティーで設定する
  double limmiter;


  // check initialize
  bool is_init;


  // for mask
  bool use_move, use_waist, use_elbow_r, use_elbow_l;
  dmatrix select_move, select_waist;


  // reference data
  vector3  dst_pos[2];
  matrix33 dst_rot[2];
  dvector  dst_redun;


  // standard model
  StandardModel* cur_model;
  StandardModel* ref_model;

public:
  void test();
};


#endif
