#ifndef __JOINTNAME_CPP__
#define __JOINTNAME_CPP__


#include "JointName.h"

using namespace tohoku;

JointName::JointName()
{
  initParameter();
}


JointName::~JointName()
{
  initParameter();
}



////////////////////////////////////////////////////////////////////
//
//    public function
//
////////////////////////////////////////////////////////////////////


bool JointName::setConf(RTC::Properties conf)
{
  //---------------------------------------------------------------------------------------
  //
  //  get data from configuration
  //
  //---------------------------------------------------------------------------------------
  
  
  // get model file
  model_file = conf.getProperty("str_model_file");
  
  
  // get DOF
  stringstream tmp;
  tmp.str(conf.getProperty("DOF"));
  tmp >> DOF;
  tmp.str("");
  // ストリームの状態をクリアする。この行がないと意図通りに動作しない
  // .str("")だけではダメみたい
  tmp.clear(stringstream::goodbit);

  
  //get real name
  for(int i = 0; i < num; i++) {
    getConfigProperty(conf, virtual_joint_name[i], real_joint_name[i], real_joint_ratio[i]);
  }


  // get offset
  getConfigProperty(conf, "R_OFFSET_R", offset_R[0]);
  getConfigProperty(conf, "R_OFFSET_P", offset_p[0]);
  getConfigProperty(conf, "L_OFFSET_R", offset_R[1]);
  getConfigProperty(conf, "L_OFFSET_P", offset_p[1]);
  


  
  //---------------------------------------------------------------------------------------
  //
  //  set other variable
  //
  //---------------------------------------------------------------------------------------  
  
  // make joint name map
  VirtualToIndexMap.clear();
  RealToIndexMap.clear();
  for(unsigned int i = 0; i < num; i++) {
    string name_virtual = virtual_joint_name[i];
    VirtualToIndexMap[name_virtual] = i;


    vector<string> name_list_real = real_joint_name[i];
    if(!name_list_real.empty()) {
      for(unsigned int j = 0; j < name_list_real.size(); j++) {
	string name_real = name_list_real[j];
	RealToIndexMap[name_real] = i;
      }
    }    
  }
  

  is_init = true;
  
  return true;
}



vector<string> JointName::getNameReal(const string name_virtual)
{
  NameToIndexMap::const_iterator p = VirtualToIndexMap.find(name_virtual);
  if(p != VirtualToIndexMap.end()) {
    return real_joint_name[p->second];
  }

  vector<string> out;
  out.clear();
  return out;
}


vector<double> JointName::getRatio(const string name_virtual)
{
  NameToIndexMap::const_iterator p = VirtualToIndexMap.find(name_virtual);
  if(p != VirtualToIndexMap.end()) {
    return real_joint_ratio[p->second];
  }
  
  vector<double> out;
  out.clear();
  return out;
}


string JointName::getNameVirtual(const string name_real)
{
  NameToIndexMap::const_iterator p = RealToIndexMap.find(name_real);
  if(p != RealToIndexMap.end()) {
    return virtual_joint_name[p->second];
  }
  
  return "";
}



vector<string> JointName::getVirtualJointNamesInPart(const string part_name)
{
  NameToIndexMap::const_iterator p = PartNameToIndexMap.find(part_name);
  if(p != PartNameToIndexMap.end()) {
    return VirtualJointNamesInPart[p->second];
  }

  vector<string> out;
  out.clear();
  return out;
}



////////////////////////////////////////////////////////////////////
//
//    private function
//
////////////////////////////////////////////////////////////////////


// for get real joint name
void JointName::getConfigProperty(RTC::Properties conf, string pro_name, vector<string> &pro, vector<double> &ratio)
{
  stringstream tmp;
  tmp.str(conf.getProperty(pro_name));

  if( !tmp.str().empty() ) {
    pro.clear();

    while(!tmp.eof()) {
      // get joint name
      string name_tmp;
      tmp >> name_tmp;

      if(name_tmp != "0") {
	// set real joint name
	pro.push_back(name_tmp);

	// set joint ratio
	if(!tmp.eof()) {
	  double ratio_tmp;
	  tmp >> ratio_tmp;
	  ratio.push_back(ratio_tmp);
	}
	else {
	  ratio.push_back(1.0);
	}
      }
      else {
	break;
      }
    }
  }
}


// for get offset
void JointName::getConfigProperty(RTC::Properties conf, string pro_name, vector<double> &pro)
{
  stringstream tmp;
  tmp.str(conf.getProperty(pro_name));

  if( !tmp.str().empty() ) {
    pro.clear();

    while(!tmp.eof()) {
      double in;
      tmp >> in;
      pro.push_back(in);
    }
  }
}



void JointName::initParameter()
{
  is_init = false;
  

  model_file = "";  
  DOF        = 0;


  // part names
  int part_num = 6;
  string part_list_tmp[] = {"MOVE", "WAIST", "RARM", "LARM", "NECK", "EXTRA"};
  VirtualJointPartNames.clear();
  PartNameToIndexMap.clear();
  for(int i = 0; i < part_num; i++) {
    VirtualJointPartNames.push_back(part_list_tmp[i]);
    PartNameToIndexMap[part_list_tmp[i]] = i;
  }
  


  string name_list_move[]  = {"BASE_X", "BASE_Y", "BASE_T"};
  string name_list_waist[] = {"WAIST1", "WAIST2", "WAIST3"};
  string name_list_rarm[]  = {"R_SHOULDER1", "R_SHOULDER2", "R_SHOULDER3", "R_ELBOW", "R_WRIST1", "R_WRIST2", "R_WRIST3"};
  string name_list_larm[]  = {"L_SHOULDER1", "L_SHOULDER2", "L_SHOULDER3", "L_ELBOW", "L_WRIST1", "L_WRIST2", "L_WRIST3"};
  string name_list_neck[]  = {"NECK1", "NECK2", "NECK3"};
  string name_list_extra[] = {"CAMERA1", "CAMERA2", "CAMERA3"};



  VirtualJointNamesInPart.clear();
  VirtualJointNamesInPart.resize(part_num);
  for(int i = 0; i < 3; i++) {
    VirtualJointNamesInPart[0].push_back(name_list_move[i]);
  }
  for(int i = 0; i < 3; i++) {
    VirtualJointNamesInPart[1].push_back(name_list_waist[i]);
  }
  for(int i = 0; i < 7; i++) {
    VirtualJointNamesInPart[2].push_back(name_list_rarm[i]);
  }
  for(int i = 0; i < 7; i++) {
    VirtualJointNamesInPart[3].push_back(name_list_larm[i]);
  }
  for(int i = 0; i < 3; i++) {
    VirtualJointNamesInPart[4].push_back(name_list_neck[i]);
  }
  for(int i = 0; i < 3; i++) {
    VirtualJointNamesInPart[5].push_back(name_list_extra[i]);
  }
  


  for(unsigned int i = 0; i < VirtualJointNamesInPart.size(); i++) {
    for(unsigned int j = 0; j < VirtualJointNamesInPart[i].size(); j++) {
      virtual_joint_name.push_back(VirtualJointNamesInPart[i][j]);
    }
  }


  real_joint_name.resize(num);
  real_joint_ratio.resize(num);
  for(unsigned int i = 0; i < num; i++) {
    real_joint_name[i].clear();
    real_joint_ratio[i].clear();
  }
  

  for(int i = 0; i < 2; i++) {
    offset_p[i].resize(3);
    offset_R[i].resize(9);

    for(int j = 0; j < 3; j++) {
      offset_p[i][j] = 0.0;
    }
    for(int j = 0; j < 9; j++) {
      offset_R[i][j] = 0.0;
    }
    offset_R[i][0] = 1.0;  offset_R[i][4] = 1.0;  offset_R[i][8] = 1.0;
  }

}


#endif
