/**********************************************************************
  標準モデル用ライブラリ

  標準モデルを構成するパーツを管理
  主に使用するであろうパーツ : MOVE, WAIST, RARM, LARM, NECK, CAMERA ... ぐらいかな？
**********************************************************************/

#ifndef STANDARD_MODEL_PART_H_INCLUDE
#define STANDARD_MODEL_PART_H_INCLUDE

#include <string>
#include <ostream>
#include <vector>
#include <map>
#include <boost/intrusive_ptr.hpp>
#include <boost/shared_ptr.hpp>


// OpenHRP3
#include <hrpModelHeaders.h>


namespace tohoku {

  class VirtualJoint;

  class StandardModelPart {

  public:
    StandardModelPart();
    StandardModelPart(std::string in_name);
    ~StandardModelPart();


    void addVirtualJoint(VirtualJoint* virtualJoint);


    void calcJacobian(OpenHRP::dmatrix& out_J, VirtualJoint* target=0) const;
    inline OpenHRP::dmatrix Jacobian(VirtualJoint* target=0) const {
      OpenHRP::dmatrix J;
      calcJacobian(J, target);
      return J;
    }


    std::string name;


    inline int numVirtualJoints() const {
      return virtualJoints.size();
    }


    inline int numRealLinks() const {
      return num_real_links;
    }


    inline int numRealJoints() const {
      return num_real_joints;
    }


    inline VirtualJoint* virtualJoint(int index) const {
      return virtualJoints[index];
    }
    VirtualJoint* virtualJoint(const std::string& name) const;


    inline VirtualJoint* rootVirtualJoint() const {
      return virtualJoints.front();
    }


    inline VirtualJoint* endVirtualJoint() const {
      return virtualJoints.back();
    }
    

  protected:
    std::vector<VirtualJoint*> virtualJoints;


  private:
    int num_real_links;
    int num_real_joints;

    typedef std::map<std::string, VirtualJoint*> NameToVirtualJointMap;
    NameToVirtualJointMap nameToVirtualJointMap;
  };

};


#endif
